@@ -30,7 +30,7 @@ class FlinkTask(object):
         proto_types = _annotated_protos_for(fun)
         self._serialiser.register_proto_types(proto_types)
 
-    async def run_async(self, task_context: TaskContext, task_request: TaskRequest):
+    async def run(self, task_context: TaskContext, task_request: TaskRequest):
         task_result, task_exception, pipeline = None, None, None
 
         task_args, kwargs, fn_state = self._to_task_args_and_kwargs(task_context, task_request)
@@ -51,22 +51,6 @@ class FlinkTask(object):
 
         return task_result, task_exception, pipeline, fn_state
 
-    def run(self, task_context: TaskContext, task_request: TaskRequest):
-        task_result, task_exception, pipeline = None, None, None
-
-        task_args, kwargs, fn_state = self._to_task_args_and_kwargs(task_context, task_request)
-
-        # run the flink task
-        try:
-            fn_result = self._fun(*task_args, **kwargs)
-            pipeline, task_result, fn_state = self._to_pipeline_or_task_result(task_request, fn_result, fn_state)
-
-        # we errored so return a task_exception instead
-        except Exception as e:
-            task_exception = self._to_task_exception(task_request, e)
-
-        return task_result, task_exception, pipeline, fn_state
-
     def _to_pipeline_or_task_result(self, task_request, fn_result, fn_state):
         pipeline, task_result, is_fruitful = None, None, self._is_fruitful
 
